package cn.sh.stc.sict.theme.hpgp.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONUtil;
import cn.sh.stc.sict.cloud.common.core.constant.Constant;
import cn.sh.stc.sict.cloud.common.core.util.NumberUtil;
import cn.sh.stc.sict.theme.hpgp.dao.HpgpBusyIdlePredictionMapper;
import cn.sh.stc.sict.theme.hpgp.dao.HpgpDepartmentRankMapper;
import cn.sh.stc.sict.theme.hpgp.model.HpgpBusyIdlePrediction;
import cn.sh.stc.sict.theme.hpgp.model.HpgpDepartmentRank;
import cn.sh.stc.sict.theme.hpgp.service.HpgpBusyIdlePredictionService;
import cn.sh.stc.sict.theme.hpgp.vo.DeptRankVO;
import cn.sh.stc.sict.theme.hphy.model.HpDeptInfo;
import cn.sh.stc.sict.theme.hphy.model.HpDocInfo;
import cn.sh.stc.sict.theme.hphy.service.HpDocInfoService;
import cn.sh.stc.sict.theme.hphy.wd.*;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 黄浦全科导诊——忙闲预测(HpgpBusyIdle_ prediction)表服务实现类
 *
 * @author zhangly
 * @since 2022-01-14 09:56:43
 */
@Slf4j
@AllArgsConstructor
@Service("hpgpBusyIdlPredictionService")
public class HpgpBusyIdlePredictionServiceImpl extends ServiceImpl<HpgpBusyIdlePredictionMapper, HpgpBusyIdlePrediction> implements HpgpBusyIdlePredictionService {
    private final HpgpDepartmentRankMapper departmentRankMapper;
    private final HpDocInfoService hpDocInfoService;

    @Override
    public List<HpgpBusyIdlePrediction> busyIdlePrediction(String hospitalCode,
                                                           String deptName) {

        // 查询当前医院有号的三个科室
        if (StrUtil.isNotBlank(hospitalCode)) {
            return busyIdlePredictionByHosCodeAndDeptName(hospitalCode, deptName);
        }

        // 在科室排名表中查询排名前三的科室展示
        return busyIdlePredictionByDeptName(deptName);
    }

    /**
     * 按科室名称查询全区排名前3的科室
     *
     * @param deptName 科室名称
     * @return HpgpBusyIdlePrediction {@link HpgpBusyIdlePrediction}
     */
    private List<HpgpBusyIdlePrediction> busyIdlePredictionByDeptName(String deptName) {
        List<DeptRankVO> rankList = departmentRankMapper.getRankByHosAndStandardDept(null, deptName, 3);
        if (CollectionUtil.isEmpty(rankList)) {
            return null;
        }

        return busyIdlePrediction(rankList);
    }


    /**
     * 查询当前医院有号的三个科室
     *
     * @param hospitalCode 医院编码
     * @param deptName     部门名称
     * @return HpgpBusyIdlePrediction {@link HpgpBusyIdlePrediction}
     */
    private List<HpgpBusyIdlePrediction> busyIdlePredictionByHosCodeAndDeptName(String hospitalCode,
                                                                                String deptName) {
        List<DeptRankVO> rankList = departmentRankMapper.getRankByHosAndStandardDept(hospitalCode, deptName, 0);
        if (CollectionUtil.isEmpty(rankList)) {
            return null;
        }

        List<HpgpBusyIdlePrediction> hpgpBusyIdlePredictionList = busyIdlePrediction(rankList);

        // 当前医院近七天有号的三个科室
        List<String> deptCodeList = hpgpBusyIdlePredictionList.stream()
                // 按科室分组，并统计每个科室可预约号源总量
                .collect(Collectors.groupingBy(HpgpBusyIdlePrediction::getDeptCode,
                        Collectors.summingInt(item -> Optional.ofNullable(item.getRemainNum()).orElse(0))))
                .entrySet().stream()
                // 按可预约号源进行排序，取排名前三的科室
                .sorted(Collections.reverseOrder(Map.Entry.comparingByValue()))
                .map(Map.Entry::getKey)
                .limit(3)
                .collect(Collectors.toList());

        return hpgpBusyIdlePredictionList.stream().filter(item -> deptCodeList.contains(item.getDeptCode())).collect(Collectors.toList());

    }

    /**
     * 按医院和科室详情查询近一周各科室忙闲情况
     *
     * @param deptList 科室集
     * @return HpgpBusyIdlePrediction {@link HpgpBusyIdlePrediction}
     */
    private List<HpgpBusyIdlePrediction> busyIdlePrediction(List<DeptRankVO> deptList) {
        DateTime startTime = DateUtil.tomorrow();
        DateTime endTime = DateUtil.offsetDay(startTime, 6);

        List<HpgpBusyIdlePrediction> result = new ArrayList<>();
        deptList.forEach(rank -> {
            HpDeptInfo dept = new HpDeptInfo();
            dept.setHosOrgCode(rank.getHospitalCode());
            dept.setHosName(rank.getHospitalName());
            dept.setOneDeptCode(rank.getOneDeptCode());
            dept.setDeptCode(rank.getDeptCode());
            dept.setDeptName(rank.getDeptName());
            // 查询医院忙闲
            result.addAll(this.listByDept(dept, startTime, endTime));
        });
        return result;
    }


    private HpgpBusyIdlePrediction getNoneSourcePredictionInfo(HpDeptInfo dept, Date i) {
        HpgpBusyIdlePrediction vo = new HpgpBusyIdlePrediction();
        vo.setOneDeptCode(dept.getOneDeptCode());
        vo.setDeptCode(dept.getDeptCode());
        vo.setDeptName(dept.getDeptName());
        vo.setHospitalName(dept.getHosName());
        vo.setHospitalCode(dept.getHosOrgCode());
        vo.setPredictionDate(i);
        vo.setScoreByNum();
        return vo;
    }

    // @Async
    @Override
    public void statisticResourceInfo(HpDeptInfo dept, Date startDate, Date endDate, Byte flag) {
        try {
            DeptInfo deptInfo = new DeptInfo();
            deptInfo.setHosOrgCode(dept.getHosOrgCode());
            deptInfo.setOneDeptCode(dept.getOneDeptCode());
            deptInfo.setDeptCode(dept.getDeptCode());
            List<OutPatInfo> outList = WanDaHttpUtil.getOutPatInfo(deptInfo);
            List<DoctInfo> docList = WanDaHttpUtil.getDoctInfo(deptInfo);
            List<HpDocInfo> hpDocList = hpDocInfoService.listByDeptInfo(deptInfo);
            if (CollUtil.isEmpty(outList) && CollUtil.isEmpty(docList) && CollUtil.isEmpty(hpDocList)) {
                return;
            }
            Map<String, HpgpBusyIdlePrediction> dateResultMap = new HashMap<>();
            this.getOutResourceInfo(dept, outList, dateResultMap, startDate, endDate);
            this.getDocResourceInfo(dept, docList, dateResultMap, startDate, endDate);
            if (Constant.BYTE_YES.equals(flag)) {
                this.getHpDocResourceInfo(dept, hpDocList, docList, dateResultMap, startDate, endDate);
            }
            if (CollUtil.isNotEmpty(dateResultMap)) {
                dateResultMap.forEach((k, v) -> {
                    LambdaQueryWrapper<HpgpBusyIdlePrediction> wrapper = new LambdaQueryWrapper<>();
                    wrapper.eq(HpgpBusyIdlePrediction::getHospitalCode, v.getHospitalCode())
                            .eq(HpgpBusyIdlePrediction::getOneDeptCode, v.getOneDeptCode())
                            .eq(HpgpBusyIdlePrediction::getDeptCode, v.getDeptCode())
                            .eq(HpgpBusyIdlePrediction::getPredictionDate, v.getPredictionDate());
                    this.saveOrUpdate(v, wrapper);
                });
            }
        } catch (Exception e) {
            log.error("统计号源异常， dept = {}", JSONUtil.toJsonStr(dept));
            log.error(e.getMessage(), e);
        }
    }

    @Override
    public List<HpgpBusyIdlePrediction>  listByDept(HpDeptInfo dept, DateTime startTime, DateTime endTime) {
        List<HpgpBusyIdlePrediction> result = new ArrayList<>();

        LambdaQueryWrapper<HpgpBusyIdlePrediction> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(HpgpBusyIdlePrediction::getHospitalCode, dept.getHosOrgCode())
                .eq(HpgpBusyIdlePrediction::getOneDeptCode, dept.getOneDeptCode())
                .eq(HpgpBusyIdlePrediction::getDeptCode, dept.getDeptCode())
                .between(HpgpBusyIdlePrediction::getPredictionDate, DateUtil.format(startTime, DatePattern.NORM_DATE_FORMAT), DateUtil.format(endTime, DatePattern.NORM_DATE_FORMAT))
                .orderByAsc(HpgpBusyIdlePrediction::getPredictionDate);
        List<HpgpBusyIdlePrediction> list = this.list(wrapper);
        if (CollUtil.isNotEmpty(list)) {
            Map<Date, HpgpBusyIdlePrediction> dateMap = list.stream().collect(Collectors.groupingBy(HpgpBusyIdlePrediction::getPredictionDate, Collectors.collectingAndThen(Collectors.toList(), x -> x.get(0))));
            // 生成无号源信息
            for (Date i = startTime; DateUtil.compare(i, endTime) < 1; i = DateUtil.offsetDay(i, 1)) {
                Date key = DateUtil.beginOfDay(i);
                if (dateMap.containsKey(key)) {
                    HpgpBusyIdlePrediction data = dateMap.get(key);
                    data.setHospitalCode(dept.getHosOrgCode());
                    data.setHospitalName(dept.getHosName());
                    data.setOneDeptCode(dept.getOneDeptCode());
                    data.setDeptCode(dept.getDeptCode());
                    data.setDeptName(dept.getDeptName());
                    result.add(data);
                } else {
                    result.add(this.getNoneSourcePredictionInfo(dept, i));
                }
            }
        } else {
            // 生成无号源信息
            for (Date i = startTime; DateUtil.compare(i, endTime) < 1; i = DateUtil.offsetDay(i, 1)) {
                result.add(this.getNoneSourcePredictionInfo(dept, i));
            }
        }

        return result;
    }

    private void getDocResourceInfo(HpDeptInfo dept,
                                    List<DoctInfo> docList,
                                    Map<String, HpgpBusyIdlePrediction> dateResultMap,
                                    Date startDate,
                                    Date endDate) {
        if (CollUtil.isNotEmpty(docList)) {
            docList.forEach(doc -> {
                NumSourceInfo numSourceInfo = new NumSourceInfo();
                numSourceInfo.setHosOrgCode(dept.getHosOrgCode());
                numSourceInfo.setOneDeptCode(dept.getOneDeptCode());
                numSourceInfo.setDeptCode(dept.getDeptCode());
                numSourceInfo.setStartTime(DateUtil.format(startDate, DatePattern.NORM_DATE_FORMAT));
                numSourceInfo.setEndTime(DateUtil.format(endDate, DatePattern.NORM_DATE_FORMAT));
                numSourceInfo.setOrderType(WanDaConstant.ORDER_TYPE_OUTP);
                numSourceInfo.setPatient_type(WanDaConstant.PATIENT_TYPE_NORMAL);
                numSourceInfo.setResourceCode(doc.getResourceCode());
                List<NumSourceInfo> orderNumInfo = WanDaHttpUtil.getOrderNumInfo(numSourceInfo);
                if (CollUtil.isNotEmpty(orderNumInfo)) {
                    hpDocInfoService.updateResource(numSourceInfo, Constant.BYTE_YES);
                    Map<String, List<NumSourceInfo>> dateMap = orderNumInfo.stream().collect(Collectors.groupingBy(NumSourceInfo::getScheduleDate));
                    dateMap.forEach((date, numList) -> {
                        HpgpBusyIdlePrediction vo = dateResultMap.getOrDefault(date, new HpgpBusyIdlePrediction());
                        vo.setDeptCode(dept.getDeptCode());
                        vo.setDeptName(dept.getDeptName());
                        vo.setHospitalName(dept.getHosName());
                        vo.setHospitalCode(dept.getHosOrgCode());
                        vo.setPredictionDate(DateUtil.parseDate(date));
                        vo.setOneDeptCode(dept.getOneDeptCode());

                        // 计算号源信息
                        int reserveOrderNum = numList.stream()
                                // 按最小粒度的预约时段分组，并取出分组后第一条号源信息
                                .collect(Collectors.groupingBy(n -> n.getNumSourceId() + n.getScheduleId(), Collectors.collectingAndThen(Collectors.toList(), value -> value.get(0))))
                                .values().stream().mapToInt(NumSourceInfo::getReserveOrderNum).sum();
                        vo.setReserveOrderNum(NumberUtil.isNullOrZero(vo.getReserveOrderNum()) ? reserveOrderNum : vo.getReserveOrderNum() + reserveOrderNum);

                        int remain = numList.stream()
                                // 按最小粒度的预约时段分组，并取出分组后第一条号源信息
                                .collect(Collectors.groupingBy(n -> n.getNumSourceId() + n.getScheduleId(), Collectors.collectingAndThen(Collectors.toList(), value -> value.get(0))))
                                .values().stream().mapToInt(NumSourceInfo::getRemainNum).sum();
                        vo.setRemainNum(NumberUtil.isNullOrZero(vo.getRemainNum()) ? remain : vo.getRemainNum() + remain);

                        vo.setScoreByNum();
                        dateResultMap.put(date, vo);
                    });
                } else {
                    hpDocInfoService.updateResource(numSourceInfo, Constant.BYTE_NO);
                }
            });
        }
    }

    private void getHpDocResourceInfo(HpDeptInfo dept,
                                      List<HpDocInfo> hpDocList, List<DoctInfo> docList,
                                      Map<String, HpgpBusyIdlePrediction> dateResultMap,
                                      Date startDate,
                                      Date endDate) {
        Set<String> resourceCodeSet = CollUtil.isEmpty(docList) ? new HashSet<>() : docList.stream().map(DoctInfo::getResourceCode).collect(Collectors.toSet());
        if (CollUtil.isNotEmpty(hpDocList)) {
            hpDocList.forEach(doc -> {
                if (StrUtil.isEmpty(doc.getResourceCode()) || resourceCodeSet.contains(doc.getResourceCode())) {
                    return;
                }
                NumSourceInfo numSourceInfo = new NumSourceInfo();
                numSourceInfo.setHosOrgCode(dept.getHosOrgCode());
                numSourceInfo.setOneDeptCode(dept.getOneDeptCode());
                numSourceInfo.setDeptCode(dept.getDeptCode());
                numSourceInfo.setStartTime(DateUtil.format(startDate, DatePattern.NORM_DATE_FORMAT));
                numSourceInfo.setEndTime(DateUtil.format(endDate, DatePattern.NORM_DATE_FORMAT));
                numSourceInfo.setOrderType(WanDaConstant.ORDER_TYPE_OUTP);
                numSourceInfo.setPatient_type(WanDaConstant.PATIENT_TYPE_NORMAL);
                numSourceInfo.setResourceCode(doc.getResourceCode());
                List<NumSourceInfo> orderNumInfo = WanDaHttpUtil.getOrderNumInfo(numSourceInfo);
                if (CollUtil.isNotEmpty(orderNumInfo)) {
                    hpDocInfoService.updateResource(numSourceInfo, Constant.BYTE_YES);
                    Map<String, List<NumSourceInfo>> dateMap = orderNumInfo.stream().collect(Collectors.groupingBy(NumSourceInfo::getScheduleDate));
                    dateMap.forEach((date, numList) -> {
                        HpgpBusyIdlePrediction vo = dateResultMap.getOrDefault(date, new HpgpBusyIdlePrediction());
                        vo.setDeptCode(dept.getDeptCode());
                        vo.setDeptName(dept.getDeptName());
                        vo.setHospitalName(dept.getHosName());
                        vo.setHospitalCode(dept.getHosOrgCode());
                        vo.setPredictionDate(DateUtil.parseDate(date));
                        vo.setOneDeptCode(dept.getOneDeptCode());

                        // 计算号源信息
                        int reserveOrderNum = numList.stream()
                                // 按最小粒度的预约时段分组，并取出分组后第一条号源信息
                                .collect(Collectors.groupingBy(n -> n.getNumSourceId() + n.getScheduleId(), Collectors.collectingAndThen(Collectors.toList(), value -> value.get(0))))
                                .values().stream().mapToInt(NumSourceInfo::getReserveOrderNum).sum();
                        vo.setReserveOrderNum(NumberUtil.isNullOrZero(vo.getReserveOrderNum()) ? reserveOrderNum : vo.getReserveOrderNum() + reserveOrderNum);

                        int remain = numList.stream()
                                // 按最小粒度的预约时段分组，并取出分组后第一条号源信息
                                .collect(Collectors.groupingBy(n -> n.getNumSourceId() + n.getScheduleId(), Collectors.collectingAndThen(Collectors.toList(), value -> value.get(0))))
                                .values().stream().mapToInt(NumSourceInfo::getRemainNum).sum();
                        vo.setRemainNum(NumberUtil.isNullOrZero(vo.getRemainNum()) ? remain : vo.getRemainNum() + remain);

                        vo.setScoreByNum();
                        dateResultMap.put(date, vo);
                    });
                } else {
                    hpDocInfoService.updateResource(numSourceInfo, Constant.BYTE_NO);
                }
            });
        }
    }

    private void getOutResourceInfo(HpDeptInfo
                                            dept, List<OutPatInfo> outList, Map<String, HpgpBusyIdlePrediction> dateResultMap, Date startDate, Date
                                            endDate) {
        if (CollUtil.isNotEmpty(outList)) {
            outList.forEach(pat -> {
                NumSourceInfo numSourceInfo = new NumSourceInfo();
                numSourceInfo.setHosOrgCode(dept.getHosOrgCode());
                numSourceInfo.setOneDeptCode(dept.getOneDeptCode());
                numSourceInfo.setDeptCode(dept.getDeptCode());
                numSourceInfo.setStartTime(DateUtil.format(startDate, DatePattern.NORM_DATE_FORMAT));
                numSourceInfo.setEndTime(DateUtil.format(endDate, DatePattern.NORM_DATE_FORMAT));
                numSourceInfo.setOrderType(WanDaConstant.ORDER_TYPE_CLINC);
                numSourceInfo.setPatient_type(WanDaConstant.PATIENT_TYPE_NORMAL);
                numSourceInfo.setResourceCode(pat.getResourceCode());
                List<NumSourceInfo> orderNumInfo = WanDaHttpUtil.getOrderNumInfo(numSourceInfo);
                if (CollUtil.isNotEmpty(orderNumInfo)) {
                    Map<String, List<NumSourceInfo>> dateMap = orderNumInfo.stream().collect(Collectors.groupingBy(NumSourceInfo::getScheduleDate));
                    dateMap.forEach((date, numList) -> {
                        HpgpBusyIdlePrediction vo = dateResultMap.getOrDefault(date, new HpgpBusyIdlePrediction());
                        vo.setDeptCode(dept.getDeptCode());
                        vo.setDeptName(dept.getDeptName());
                        vo.setHospitalName(dept.getHosName());
                        vo.setHospitalCode(dept.getHosOrgCode());
                        vo.setPredictionDate(DateUtil.parseDate(date));
                        vo.setOneDeptCode(dept.getOneDeptCode());

                        // 计算号源信息
                        int reserveOrderNum = numList.stream()
                                // 按最小粒度的预约时段分组，并取出分组后第一条号源信息
                                .collect(Collectors.groupingBy(n -> n.getNumSourceId() + n.getScheduleId(), Collectors.collectingAndThen(Collectors.toList(), value -> value.get(0))))
                                .values().stream().mapToInt(NumSourceInfo::getReserveOrderNum).sum();
                        vo.setReserveOrderNum(NumberUtil.isNullOrZero(vo.getReserveOrderNum()) ? reserveOrderNum : vo.getReserveOrderNum() + reserveOrderNum);

                        int remain = numList.stream()
                                // 按最小粒度的预约时段分组，并取出分组后第一条号源信息
                                .collect(Collectors.groupingBy(n -> n.getNumSourceId() + n.getScheduleId(), Collectors.collectingAndThen(Collectors.toList(), value -> value.get(0))))
                                .values().stream().mapToInt(NumSourceInfo::getRemainNum).sum();
                        vo.setRemainNum(NumberUtil.isNullOrZero(vo.getRemainNum()) ? remain : vo.getRemainNum() + remain);

                        vo.setScoreByNum();
                        dateResultMap.put(date, vo);
                    });
                }
            });
        }
    }
}
